@@ -301,7 +301,35 @@ def amp_phase(h):
 
 
 def phase(h):
-  """Get phase of waveform, h = A*exp(i*phi)"""
+  """Get phase of waveform, h = A*exp(i*phi). 
+  
+  Preserves shape ( np.shape(h) == np.shape(phase) ) and gracefully handles h=0
+  
+  Input
+  =====
+  h - numpy vector or array of shape (N,1) or (N,)
+  
+  
+  Examples
+  >>> t1 = np.linspace(0,10,100)
+  >>> y1 = np.exp(1.0j*t1)
+  >>> phase1 = phase(y1)
+  >>> y1.shape
+  (100,)
+  >>> phase1.shape
+  (100,)
+  >>> t3 = np.reshape(t1,(100,1))
+  >>> y3 = np.exp(1.0j*t3)
+  >>> phase3 = phase(y3)
+  >>> phase3.shape
+  (100,1)
+  >>> y3.shape
+  (100,1)
+  """
+  
+  if len(h.shape) == 2:
+      assert(h.shape[0] > h.shape[1] ), 'input array expected to have shape like (N,1) and not (1,N)'
+  
   if np.shape(h):
     # Compute the phase only for non-zero values of h, otherwise set phase to zero.
     nonzero_h = h[np.abs(h) > 1e-300]
@@ -310,7 +338,7 @@ def phase(h):
   else:
     nonzero_h = h
     phase = np.real(-1j*np.log(nonzero_h/np.abs(nonzero_h)))
-  return phase
+  return np.reshape(phase,h.shape)
 
 
 def cycles(h):
@@ -1582,3 +1610,73 @@ def modes_list_to_dict(mode_list, mode_array):
     else:
         mode_dict[mode] = mode_array[i,:]
   return mode_dict
+
+
+
+#### convenient wrapper to sxs memory interface ###
+
+def sxs_memory(h_lm, times, verbose=False):
+  """  Compute memory modes using sxs module. Make sure sxs is installed as
+  
+    >>> conda install -c conda-forge sxs
+  
+  Input
+  =====
+  h_lm: dictionary of oscillatory modes. For example,
+  
+            hdict_tmp[(2,2)] = np.array( [...] )
+  times: array of times defining the hdict_tmp modes
+  verbose: Output verbosity
+  
+  Output
+  ======
+  memory_sxs: dictionary of memory modes
+  
+  """
+
+  import sxs
+
+  # find the maximum/min value of ell stored in hdict_tmp
+  max_ell=-1
+  min_ell=100
+  last_mode = None
+  for mode in h_lm.keys():
+    ell = mode[0]
+    if ell > max_ell:
+      max_ell = ell
+    if ell < min_ell:
+      min_ell = ell
+    last_mode = mode
+
+  assert(last_mode is not None)
+
+  if verbose:
+    print("max ell = %i, min ell= %i"%(max_ell, min_ell))
+
+  # compute x2 more memory modes than oscillatory modes
+  max_ell = 2*max_ell
+
+  modes = [(ell, m) for ell in range(2, max_ell+1) for m in range(-ell,ell+1)]
+  sxs_modes = []
+  for mode in modes:
+    if verbose:
+      print(mode)
+    if mode in h_lm.keys():
+      sxs_modes.append( h_lm[mode] )
+    else:
+      sxs_modes.append( np.zeros_like(h_lm[last_mode]) )
+  sxs_modes = np.array( sxs_modes )
+
+  h_test = sxs.waveforms.WaveformModes(sxs_modes.transpose(), time=times, modes_axis=1, time_axis=0, ell_min=min_ell, ell_max=max_ell)
+  h_test._metadata['spin_weight'] = -2
+
+  # WARNING: hardcoded start time
+  h_with_memory = sxs.waveforms.memory.add_memory(h_test, start_time=times[0])
+
+  memory_sxs = {}
+  for mode in modes:
+    ell=mode[0]
+    emm=mode[1]
+    memory_sxs[mode] = np.array( h_with_memory[:,h_with_memory.index(ell,emm)] - h_test[:,h_test.index(ell,emm)] )
+        
+  return memory_sxs, times
